package com.fr.plugin.xx.ltqc.auth.action;

import com.fr.plugin.db.PluginDBManager;
import com.fr.plugin.xx.ltqc.auth.AuthDBAccessProvider;
import com.fr.plugin.xx.ltqc.auth.dao.RolePathAuthDao;
import com.fr.plugin.xx.ltqc.auth.entity.RolePathAuthEntity;
import com.fr.plugin.xx.ltqc.auth.utils.LogUtils;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.BaseDAO;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.db.session.DBSession;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.restriction.RestrictionFactory;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Author xx
 * @Date 2022/2/14
 * @Description
 **/
public class RolePathAuthService {

    public static void save(List<RolePathAuthEntity> list) {
        try {
            AuthDBAccessProvider.getDbAccessor().runDMLAction((DBAction<Integer>) daoContext -> {
                save(daoContext.getDAO(RolePathAuthDao.class), list);
                return null;
            });
        } catch (Exception e) {
            LogUtils.debug4plugin(e.getMessage(), e);
        }

    }


    private static void save(BaseDAO dao, List<RolePathAuthEntity> list) {
        try {
            if (list == null || list.isEmpty()) {
                return;
            }
            DBSession session = PluginDBManager.getInstance().getDbContext().openSession();
            session.beginTransaction();//开始事务
            Set<String> set = list.stream().map(RolePathAuthEntity::getRole).collect(Collectors.toSet());
            dao.remove(QueryFactory.create().addRestriction(RestrictionFactory.in("role", set)));
            for (RolePathAuthEntity e : list) {
                dao.addOrUpdate(e);
            }
            session.commitTransaction();//提交
            session.closeSession();
        } catch (Exception e) {
            LogUtils.debug4plugin(e.getMessage(), e);
        }

    }

    public static Integer delete(String role) {
        try {
            return AuthDBAccessProvider.getDbAccessor().runDMLAction(new DBAction<Integer>() {
                @Override
                public Integer run(DAOContext daoContext) throws Exception {
                    daoContext.getDAO(RolePathAuthDao.class).remove(QueryFactory.create().addRestriction(RestrictionFactory.eq("role", role)));
                    return 1;
                }
            });
        } catch (Exception e) {
            LogUtils.debug4plugin(e.getMessage(), e);
        }
        return 1;
    }

    public static Set<String> getPathsByRole(Set<String> ids) {
        try {
            return AuthDBAccessProvider.getDbAccessor().runDMLAction(daoContext -> {
                List<RolePathAuthEntity> list = daoContext.getDAO(RolePathAuthDao.class).find(QueryFactory.create().addRestriction(RestrictionFactory.in("role", ids)));
                if (list == null || list.isEmpty()) {
                    return new HashSet<>();
                }
                return list.stream().map(RolePathAuthEntity::getPath).collect(Collectors.toSet());
            });
        } catch (Exception e) {
            LogUtils.debug4plugin(e.getMessage(), e);
        }
        return new HashSet();
    }
}
