package com.fr.plugin.xx.ltqc.auth.handler;

import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.base.constant.type.operation.ManualOperationType;
import com.fr.decision.authority.data.CustomRole;
import com.fr.decision.authority.data.User;
import com.fr.decision.base.util.UUIDUtil;
import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.decision.privilege.encrpt.PasswordValidator;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.decision.webservice.bean.user.UserUpdateBean;
import com.fr.decision.webservice.utils.UserSourceFactory;
import com.fr.decision.webservice.v10.user.CustomRoleService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.json.JSONObject;
import com.fr.plugin.xx.ltqc.auth.utils.LogUtils;
import com.fr.stable.StringUtils;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.util.List;
import java.util.UUID;

/**
 * @Author xx
 * @Date 2022/2/13
 * @Description
 **/
public class AddUserHandler extends BaseHttpHandler {


    @Override
    public RequestMethod getMethod() {
        return RequestMethod.POST;
    }

    @Override
    public String getPath() {
        return "/auth/addUser";
    }

    @Override
    public boolean isPublic() {
        return false;
    }

    @Override
    public void handle(HttpServletRequest req, HttpServletResponse res) {
        try {
            res.setContentType("application/json; charset=UTF-8");
            JSONObject body = parseRequest(req);
            String admin = UserService.getInstance().getAdminUserIdList().get(0);
            if (body == null || body.isEmpty()) {
                WebUtils.printAsJSON(res, error("body is not null"));
                return;
            }
            String id = body.getString("id");
            User getUser = UserService.getInstance().getUserByUserId(id);
            UserBean userBean = new UserBean();
            userBean.setUsername(body.getString("userName"));
            userBean.setRealName(body.getString("realName"));
            userBean.setEmail(body.getString("email"));
            userBean.setMobile(body.getString("mobile"));
            userBean.setId(id);
            String roleId = body.getString("roleId");
            if (StringUtils.isNotBlank(roleId)) {
                userBean.setRoleIds(roleId.split(","));
            }
            User user1 = (new User()).id(userBean.getId()).userName(userBean.getUsername()).realName(userBean.getRealName()).email(userBean.getEmail()).mobile(userBean.getMobile()).creationType(ManualOperationType.KEY).lastOperationType(ManualOperationType.KEY).enable(true);
            if (getUser == null) {
                userBean.setPassword(UUID.randomUUID().toString());
                String uuid = UUIDUtil.generate();
                PasswordValidator validator = UserSourceFactory.getInstance().getUserSource(ManualOperationType.KEY).getPasswordValidator();
                user1.password(validator.encode(userBean.getUsername(), userBean.getPassword(), uuid)).salt(uuid);
                AuthorityContext.getInstance().getUserController().add(user1);
            } else {
                getUser.userName(userBean.getUsername()).realName(userBean.getRealName()).email(userBean.getEmail()).mobile(userBean.getMobile()).creationType(ManualOperationType.KEY).lastOperationType(ManualOperationType.KEY).enable(true);
                AuthorityContext.getInstance().getUserController().update(getUser);
            }
            LogUtils.debug4plugin("delete roles by user is {}",user1.getId());
            List<CustomRole> customRolesByUser = CustomRoleService.getInstance().getCustomRolesByUser(user1.getId());
            for (CustomRole role : customRolesByUser){
                UserUpdateBean userUpdateBean = new UserUpdateBean();
                userUpdateBean.setRemoveUserIds(new String[]{user1.getId()});
                UserService.getInstance().updateRoleUsers(role.getId(), userUpdateBean);
            }
            /*AuthDBAccessProvider.getDbAccessor().runQueryAction((DBAction<Integer>) daoContext -> {
                daoContext.getDAO(UserRoleMiddleDAO.class).remove(QueryFactory.create().addRestriction(RestrictionFactory.eq("userId", user1.getId())).addRestriction(RestrictionFactory.eq("roleType", RoleType.CUSTOM)));
                return null;
            });*/
            UserService.getInstance().updateUserRoles(admin, userBean);
            WebUtils.printAsJSON(res, JSONObject.create().put("state", 0));
        } catch (Exception e) {
            LogUtils.error(e.getMessage(), e);
            try {
                WebUtils.printAsJSON(res, error(e.getMessage()));
            } catch (Exception ex) {
                LogUtils.error(ex.getMessage(), ex);
            }
        }
    }

    private JSONObject error(String mess) {
        return JSONObject.create().put("state", 1).put("mess", mess);
    }

    private JSONObject parseRequest(HttpServletRequest request) {
        try {
            BufferedReader br = request.getReader();
            String str = "";
            String listString = "";
            while ((str = br.readLine()) != null) {
                listString += str;
            }
            return new JSONObject(listString);
        } catch (Exception e) {
            return new JSONObject();
        }
    }
}
