package com.fr.plugin.xx.ltqc.auth.handler;

import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.base.constant.type.operation.ManualOperationType;
import com.fr.decision.authority.data.CustomRole;
import com.fr.decision.authority.data.User;
import com.fr.decision.base.util.UUIDUtil;
import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.decision.privilege.encrpt.PasswordValidator;
import com.fr.decision.webservice.bean.user.RoleBean;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.decision.webservice.bean.user.UserUpdateBean;
import com.fr.decision.webservice.utils.UserSourceFactory;
import com.fr.decision.webservice.v10.user.CustomRoleService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.io.utils.ResourceIOUtils;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.xx.ltqc.auth.Constants;
import com.fr.plugin.xx.ltqc.auth.action.RolePathAuthService;
import com.fr.plugin.xx.ltqc.auth.entity.RolePathAuthEntity;
import com.fr.plugin.xx.ltqc.auth.utils.LogUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.data.DataList;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @Author xx
 * @Date 2022/2/13
 * @Description
 **/
public class EditRoleHandler extends BaseHttpHandler {


    @Override
    public RequestMethod getMethod() {
        return RequestMethod.POST;
    }

    @Override
    public String getPath() {
        return "/auth/editRole";
    }

    @Override
    public boolean isPublic() {
        return false;
    }

    @Override
    public void handle(HttpServletRequest req, HttpServletResponse res) {
        try {
            res.setContentType("application/json; charset=UTF-8");
            JSONObject body = parseRequest(req);
            String admin = UserService.getInstance().getAdminUserIdList().get(0);
            User user = UserService.getInstance().getUserByRequestCookie(req);
            if (body == null || body.isEmpty()) {
                WebUtils.printAsJSON(res, error("body is not null"));
                return;
            }
            if (!body.has("roles")) {
                WebUtils.printAsJSON(res, error("role is not null"));
                return;
            }
            JSONObject role = body.getJSONObject("roles");
            RoleBean roleBean = new RoleBean();
            roleBean.setText(role.getString("text"));
            roleBean.setId(role.getString("id"));
            roleBean.setDescription(role.getString("description"));
            RoleBean customRole = CustomRoleService.getInstance().getCustomRole(roleBean.getId());
            CustomRole customRole1 = (new CustomRole()).id(roleBean.getId()).name(roleBean.getText()).description(roleBean.getDescription()).creationType(ManualOperationType.KEY).lastOperationType(ManualOperationType.KEY).enable(true);
            if (customRole == null) {
                AuthorityContext.getInstance().getCustomRoleController().add(customRole1);
            } else {
                AuthorityContext.getInstance().getCustomRoleController().update(customRole1);
            }
            //删除所有的用户
            LogUtils.debug4plugin("delete roles by roles is {}",customRole.getId());
            DataList<User> roleUser = AuthorityContext.getInstance().getUserController().findByCustomRole(customRole.getId(), QueryFactory.create());
            UserUpdateBean userUpdateBean = new UserUpdateBean();
            userUpdateBean.setRemoveUserIds(roleUser.getList().stream().map(User::getId).collect(Collectors.toList()).toArray(new String[0]));
            UserService.getInstance().updateRoleUsers(customRole.getId(), userUpdateBean);

            JSONArray users = body.getJSONArray("users");
            for (int i = 0; i < users.size(); i++) {
                JSONObject object = users.getJSONObject(i);
                String id = object.getString("id");
                User getUser = UserService.getInstance().getUserByUserId(id);
                UserBean userBean = new UserBean();
                userBean.setUsername(object.getString("userName"));
                userBean.setRealName(object.getString("realName"));
                userBean.setEmail(object.getString("email"));
                userBean.setMobile(object.getString("mobile"));
                userBean.setId(id);
                List<RoleBean> roles = UserService.getInstance().getTargetUserRoles(admin, id);
                if (roles != null) {
                    List<String> ids = roles.stream().map(RoleBean::getId).collect(Collectors.toList());
                    ids.add(roleBean.getId());
                    userBean.setRoleIds(ids.toArray(new String[0]));
                } else {
                    userBean.setRoleIds(new String[]{roleBean.getId()});
                }
                User user1 = (new User()).id(userBean.getId()).userName(userBean.getUsername()).realName(userBean.getRealName()).email(userBean.getEmail()).mobile(userBean.getMobile()).creationType(ManualOperationType.KEY).lastOperationType(ManualOperationType.KEY).enable(true);
                if (getUser == null) {
                    userBean.setPassword(UUID.randomUUID().toString());
                    String uuid = UUIDUtil.generate();
                    PasswordValidator validator = UserSourceFactory.getInstance().getUserSource(ManualOperationType.KEY).getPasswordValidator();
                    user1.password(validator.encode(userBean.getUsername(), userBean.getPassword(), uuid)).salt(uuid);
                    AuthorityContext.getInstance().getUserController().add(user1);
                } else {
                    getUser.userName(userBean.getUsername()).realName(userBean.getRealName()).email(userBean.getEmail()).mobile(userBean.getMobile()).creationType(ManualOperationType.KEY).lastOperationType(ManualOperationType.KEY).enable(true);
                    AuthorityContext.getInstance().getUserController().update(getUser);
                }
                UserService.getInstance().updateUserRoles(admin, userBean);
            }

            JSONArray paths = body.getJSONArray("paths");
            List<RolePathAuthEntity> saves = new ArrayList<>();
            for (int j = 0; j < paths.size(); j++) {
                JSONObject path = paths.getJSONObject(j);
                String realPath = Constants.ROOT + path.getString("path");
                if (!ResourceIOUtils.exist(realPath)) {
                    WebUtils.printAsJSON(res, error(String.format("cpt %s not exist", path)));
                    return;
                }
                RolePathAuthEntity entity = new RolePathAuthEntity();
                entity.setId(UUID.randomUUID().toString());
                entity.setRole(roleBean.getId());
                entity.setPath(path.getString("path"));
                entity.setCreateUser(user.getId());
                entity.setCreateTime(new Date());
                saves.add(entity);
            }
            RolePathAuthService.save(saves);
            WebUtils.printAsJSON(res, JSONObject.create().put("state", 0));
        } catch (Exception e) {
            LogUtils.error(e.getMessage(), e);
            try {
                WebUtils.printAsJSON(res, error(e.getMessage()));
            } catch (Exception ex) {
                LogUtils.error(ex.getMessage(), ex);
            }
        }
    }

    private JSONObject error(String mess) {
        return JSONObject.create().put("state", 1).put("mess", mess);
    }

    private JSONObject parseRequest(HttpServletRequest request) {
        try {
            BufferedReader br = request.getReader();
            String str = "";
            String listString = "";
            while ((str = br.readLine()) != null) {
                listString += str;
            }
            return new JSONObject(listString);
        } catch (Exception e) {
            return new JSONObject();
        }
    }
}
